@@ -557,11 +557,11 @@ def where(condition, x, y):
     """
     return choose(not_equal(condition, 0), (y, x))
 
-def compress(condition, m, dimension=-1):
-    """compress(condition, x, dimension=-1) = those elements of x corresponding 
+def compress(condition, m, axis=-1):
+    """compress(condition, x, axis=-1) = those elements of x corresponding 
     to those elements of condition that are "true".  condition must be the
     same size as the given dimension of x."""
-    return take(m, nonzero(condition), dimension)
+    return take(m, nonzero(condition), axis)
 
 def clip(m, m_min, m_max):
     """clip(m, m_min, m_max) = every entry in m that is less than m_min is
@@ -592,7 +592,7 @@ def sum (x, axis=0):
     if axis < 0: axis += n
     if n == 0 and axis in [0,-1]: return x[0]
     if axis < 0 or axis >= n:
-        return ValueError, 'Improper axis argument to sum.'
+        raise ValueError, 'Improper axis argument to sum.'
     return add.reduce(x, axis)
 
 def product (x, axis=0):
@@ -645,7 +645,7 @@ def cumproduct (x, axis=0):
         return ValueError, 'Improper axis argument to cumproduct.'
     return multiply.accumulate(x, axis)
 
-arange = arrayrange
+arange = multiarray.arange
 
 def around(m, decimals=0):
     """around(m, decimals=0) \
@@ -776,3 +776,15 @@ def average (a, axis=0, weights=None, returned = 0):
         return n/d, d
     else:
         return n/d
+
+def vdot(a, b):
+    """Returns the dot product of 2 vectors (or anything that can be made into
+       a vector). NB: this is not the same as `dot`, as it takes the conjugate
+       of its first argument if complex and always returns a scalar."""
+    return multiarray.matrixproduct(conjugate(ravel(a)),
+                                    ravel(b))
+
+# try to import blas optimized dot, innerproduct and vdot, if available
+try:
+    from dotblas import dot, innerproduct, vdot
+except ImportError: pass
