@@ -13,9 +13,10 @@
   performance for heterogeneity.
 */
 
-/* $Id: arrayobject.c,v 1.49 2002/03/05 18:10:22 dubois Exp $ */
+/* $Id: arrayobject.c,v 1.68 2004/06/07 21:07:07 teoliphant Exp $ */
 
 #include "Python.h"
+#include <stddef.h>
 /*Silly trick to make dll library work right under Win32*/
 #if defined(MS_WIN32) || defined(__CYGWIN__)
 #undef DL_IMPORT
@@ -32,7 +33,7 @@
 /* allocated statically.  This is the size of that static allocation.  I */
 /* can't come up with any reasonable excuse for a larger array than this. */
 
-#define MAX_DIMS 40
+#define MAX_DIMS 30
 
 /* Helper Functions */
 extern int _PyArray_multiply_list(int *l1, int n) {
@@ -382,6 +383,7 @@ PyObject *PyArray_FromDimsAndDataAndDescr(int nd, int *d,
     self->descr=descr;
     self->base = (PyObject *)NULL;
     self->flags = flags;
+    self->weakreflist = (PyObject *)NULL;
 	
     return (PyObject*)self;
 	
@@ -449,7 +451,7 @@ static PyObject * PyArray_Resize(PyArrayObject *self, PyObject *shape) {
     }
 
     refcnt = (((PyObject *)self)->ob_refcnt);
-    if ((refcnt > 2) || (self->base != NULL)) {
+    if ((refcnt > 2) || (self->base != NULL) || (self->weakreflist != NULL)) {
 	PyErr_SetString(PyExc_ValueError, "cannot resize an array that has been referenced or is referencing\n  another array in this way.  Use the resize function.");
 	return NULL;
     }
@@ -555,6 +557,10 @@ static PyObject * PyArray_Resize(PyArrayObject *self, PyObject *shape) {
 }
 
 static void array_dealloc(PyArrayObject *self) {
+
+    if (self->weakreflist != NULL)
+        PyObject_ClearWeakRefs((PyObject *)self);
+
     if(self->base) {
 	Py_DECREF(self->base);
     }
@@ -622,8 +628,13 @@ extern PyObject *PyArray_Return(PyArrayObject *mp) {
 	if (mp != NULL) {Py_DECREF(mp);}
 	return NULL;
     }
-    if (mp->nd == 0 && (mp->descr->type_num == PyArray_LONG || mp->descr->type_num == PyArray_DOUBLE || mp->descr->type_num == PyArray_CDOUBLE || mp->descr->type_num == PyArray_OBJECT )) {
-	op = mp->descr->getitem(mp->data);
+    if (mp->nd == 0) {
+	if (mp->descr->type_num == PyArray_LONG || mp->descr->type_num == PyArray_DOUBLE || mp->descr->type_num == PyArray_CDOUBLE || mp->descr->type_num == PyArray_OBJECT ) {
+	    op = mp->descr->getitem(mp->data);
+	}
+        else {
+	    op = PyArray_Copy(mp);
+	}
 	Py_DECREF(mp);
 	return op;
     } else {
@@ -676,8 +687,8 @@ static int array_ass_item(PyArrayObject *self, int i, PyObject *v) {
     PyArrayObject *tmp;
     char *item;
     int ret;
-	
-    if (v == NULL) {
+
+     if (v == NULL) {
 	PyErr_SetString(PyExc_ValueError, "Can't delete array elements.");
 	return -1;
     }
@@ -688,11 +699,11 @@ static int array_ass_item(PyArrayObject *self, int i, PyObject *v) {
 	if((tmp = (PyArrayObject *)array_item(self, i)) == NULL) return -1;
 	ret = PyArray_CopyObject(tmp, v);
 	Py_DECREF(tmp);
-	return ret;
+	return ret;   
     }
 	
     if ((item = index2ptr(self, i)) == NULL) return -1;
-	
+
     if(self->descr->type_num != PyArray_OBJECT && PyString_Check(v) && PyObject_Length(v) == 1) {
 	char *s;
 	if ((s=PyString_AsString(v)) == NULL) return -1;
@@ -703,11 +714,10 @@ static int array_ass_item(PyArrayObject *self, int i, PyObject *v) {
 	if(s) c=PyInt_FromLong((long)*s);
 	if(c) v=c;
     }
-	
+
     self->descr->setitem(v, item);
-	
     if(c) {Py_DECREF(c);}
-    if(PyErr_Occurred()) {return -1;}
+    if(PyErr_Occurred()) return -1;
     return 0;
 }
 
@@ -751,8 +761,11 @@ static int slice_GetIndices(PySliceObject *r, int length,
 	*stop = PyInt_AsLong(r->stop);
 	if (*stop < 0) *stop += length;
     }
-
-    if (*start > (length-1)) *start = length;
+    if (*step < 0) {
+        if (*start > (length-1)) *start = length-1;
+    } else {
+        if (*start > length) *start = length;
+    }
     if (*start < 0) *start = 0;
     if (*stop < -1) *stop = -1;
     else if (*stop > length) *stop = length;
@@ -820,6 +833,7 @@ static int parse_subindex(PyObject *op, int *step_size, int *n_steps, int max) {
 	}
 	return tmp;
     } 
+
     PyErr_SetString(PyExc_IndexError, 
 		    "each subindex must be either a slice, an integer, Ellipsis, or NewAxis");
     return -1;
@@ -832,14 +846,16 @@ static int parse_index(PyArrayObject *self, PyObject *op,
     int step_size, n_steps;
     PyObject *op1=NULL;
     int is_slice;
-	
+
+
     if (PySlice_Check(op) || op == Py_Ellipsis) {
 	n = 1;
 	op1 = op;
 	Py_INCREF(op);  
 	/* this relies on the fact that n==1 for loop below */
 	is_slice = 1;
-    } else {
+    }
+    else {
 	if (!PySequence_Check(op)) {
 	    PyErr_SetString(PyExc_IndexError, 
 			    "index must be either an int or a sequence");
@@ -851,6 +867,10 @@ static int parse_index(PyArrayObject *self, PyObject *op,
 	
     nd_old = nd_new = 0;
 	
+    /* IF we have at least two args with the last being a FLAG|FUNCTION
+        do me...
+    */
+
     offset = 0;
     for(i=0; i<n; i++) {
 	if (!is_slice) {
@@ -859,7 +879,7 @@ static int parse_index(PyArrayObject *self, PyObject *op,
 		return -1;
 	    }
 	}
-		
+	
 	start = parse_subindex(op1, &step_size, &n_steps, 
 			       nd_old < self->nd ? self->dimensions[nd_old] : 0);
 	Py_DECREF(op1);
@@ -1064,7 +1084,13 @@ static int array_getwritebuf(PyArrayObject *self, int segment, void **ptrptr) {
 }
 
 static int array_getcharbuf(PyArrayObject *self, int segment, const char **ptrptr) {
-    return array_getreadbuf(self, segment, (void **) ptrptr);
+    if (self->descr->type_num == PyArray_CHAR)
+        return array_getreadbuf(self, segment, (void **) ptrptr);
+    else {
+        PyErr_SetString(PyExc_TypeError, 
+                        "Non-character array cannot be interpreted as character buffer.");
+        return -1;
+    }
 }
 
 static PyBufferProcs array_as_buffer = {
@@ -1319,6 +1345,7 @@ static PyObject *array_inplace_true_divide(PyArrayObject *m1, PyObject *m2) {
 }
 /*End of methods added by Bruce Sherwood*/
 
+/* Array evaluates as "true" if any of the elements are non-zero */
 static int array_nonzero(PyArrayObject *mp) {
     char *zero;
     PyArrayObject *self;
@@ -1336,7 +1363,7 @@ static int array_nonzero(PyArrayObject *mp) {
     }
 	
     Py_DECREF(self);
-	
+
     return i!=s;
 }
 
@@ -1360,8 +1387,8 @@ static PyObject *array_divmod(PyArrayObject *op1, PyObject *op2) {
 /* methods added by Travis Oliphant, Jan 2000 */
 static PyObject *array_int(PyArrayObject *v) {        
     PyObject *pv, *pv2;
-    if (v->nd != 0) {
-	PyErr_SetString(PyExc_TypeError, "Only rank-0 arrays can be converted to Python scalars.");
+    if (PyArray_SIZE(v) != 1) {
+	PyErr_SetString(PyExc_TypeError, "only length-1 arrays can be converted to Python scalars.");
 	return NULL;
     }
     pv = v->descr->getitem(v->data);
@@ -1382,10 +1409,10 @@ static PyObject *array_int(PyArrayObject *v) {
     return pv2;        
 }
 
-static PyObject *array_float(PyArrayObject *v) {        
+static PyObject *array_float(PyArrayObject *v) {
     PyObject *pv, *pv2;
-    if (v->nd != 0) {
-	PyErr_SetString(PyExc_TypeError, "only rank-0 arrays can be converted to Python scalars.");
+    if (PyArray_SIZE(v) != 1) {
+	PyErr_SetString(PyExc_TypeError, "only length-1 arrays can be converted to Python scalars.");
 	return NULL;
     }
     pv = v->descr->getitem(v->data);
@@ -1407,8 +1434,8 @@ static PyObject *array_float(PyArrayObject *v) {
 
 static PyObject *array_long(PyArrayObject *v) {        
     PyObject *pv, *pv2;
-    if (v->nd != 0) {
-	PyErr_SetString(PyExc_TypeError, "only rank-0 arrays can be converted to Python scalars.");
+    if (PyArray_SIZE(v) != 1) {
+	PyErr_SetString(PyExc_TypeError, "only length-1 arrays can be converted to Python scalars.");
 	return NULL;
     }
     pv = v->descr->getitem(v->data);
@@ -1427,8 +1454,8 @@ static PyObject *array_long(PyArrayObject *v) {
 
 static PyObject *array_oct(PyArrayObject *v) {        
     PyObject *pv, *pv2;
-    if (v->nd != 0) {
-	PyErr_SetString(PyExc_TypeError, "only rank-0 arrays can be converted to Python scalars.");
+    if (PyArray_SIZE(v) != 1) {
+	PyErr_SetString(PyExc_TypeError, "only length-1 arrays can be converted to Python scalars.");
 	return NULL;
     }
     pv = v->descr->getitem(v->data);
@@ -1447,8 +1474,8 @@ static PyObject *array_oct(PyArrayObject *v) {
 
 static PyObject *array_hex(PyArrayObject *v) {        
     PyObject *pv, *pv2;
-    if (v->nd != 0) {
-	PyErr_SetString(PyExc_TypeError, "only rank-0 arrays can be converted to Python scalars.");
+    if (PyArray_SIZE(v) != 1) {
+	PyErr_SetString(PyExc_TypeError, "only length-1 arrays can be converted to Python scalars.");
 	return NULL;
     }
     pv = v->descr->getitem(v->data);
@@ -1507,9 +1534,9 @@ static PyNumberMethods array_as_number = {
     (binaryfunc)array_inplace_bitwise_xor,  /*inplace_xor*/
     (binaryfunc)array_inplace_bitwise_or,   /*inplace_or*/
 
-	/* Added in release 2.2 */
+        /* Added in release 2.2 */
 	/* The following require the Py_TPFLAGS_HAVE_CLASS flag */
-#ifdef Py_TPFLAGS_HAVE_CLASS
+#if PY_VERSION_HEX >= 0x02020000
 	(binaryfunc)array_floor_divide,          /*nb_floor_divide*/
 	(binaryfunc)array_true_divide,           /*nb_true_divide*/
 	(binaryfunc)array_inplace_floor_divide,  /*nb_inplace_floor_divide*/
@@ -1530,7 +1557,7 @@ static PySequenceMethods array_as_sequence = {
 /* -------------------------------------------------------------- */
 
 /*This ugly function quickly prints out an array.  It's likely to disappear */
-/*in the near future. */
+/*in the near future. (or maybe not...) */
 static int dump_data(char **string, int *n, int *max_n, 
                      char *data, int nd, int *dimensions, 
                      int *strides, PyArray_Descr *descr) {
@@ -1787,7 +1814,10 @@ PyObject *array_cast(PyArrayObject *self, PyObject *args) {
     if (!PyArg_ParseTuple(args, "O", &op)) return NULL;
 	
     if (PyString_Check(op) && (PyString_Size(op) == 1)) {
-	return PyArray_Cast(self, PyString_AS_STRING((PyStringObject *)op)[0]);
+        typecode = PyString_AS_STRING((PyStringObject *)op)[0];
+        if (PyArray_ValidType(typecode)) return PyArray_Cast(self, typecode);
+        PyErr_SetString(PyExc_ValueError, "Invalid type for array");
+        return NULL;
     }
 	
     if (PyType_Check(op)) {
@@ -2036,9 +2066,10 @@ static PyObject *array_getattr(PyArrayObject *self, char *name) {
 							   self->dimensions,
 							   self->descr->type_num,
 							   self->data);
-	    if (ret == NULL) return NULL;
+	    if (ret == NULL) return NULL; 
 	    Py_INCREF(self);
 	    ret->base = (PyObject *)self;
+            return (PyObject *)ret;
 	}
     }
 	
@@ -2171,11 +2202,12 @@ PyTypeObject PyArray_Type = {
     (setattrofunc)0L,                     /*tp_setattro*/
     (PyBufferProcs *)&array_as_buffer,    /*tp_as_buffer*/
     (Py_TPFLAGS_HAVE_INPLACEOPS | 
-#if PY_VERSION_HEX >= 0x02200000
+#if PY_VERSION_HEX >= 0x02020000
          Py_TPFLAGS_HAVE_CLASS |
 #endif
 #if PY_VERSION_HEX >= 0x02010000
 	 Py_TPFLAGS_HAVE_RICHCOMPARE |
+         Py_TPFLAGS_HAVE_WEAKREFS |
 #endif
      Py_TPFLAGS_HAVE_GETCHARBUFFER),      /*tp_flags*/
     /*Documentation string */
@@ -2185,6 +2217,7 @@ PyTypeObject PyArray_Type = {
     (traverseproc)0L,
     (inquiry)0L,
     (richcmpfunc)array_richcompare,       /*tp_richcompfunc*/
+    offsetof(PyArrayObject, weakreflist), /*tp_weaklistoffset */
 #endif
 };
 
@@ -2203,6 +2236,7 @@ static int discover_depth(PyObject *s, int max, int stop_at_string) {
     if(! PySequence_Check(s) || PySequence_Length(s) < 0) {
 	PyErr_Clear(); return 0;
     }
+    if (PyArray_Check(s) && (((PyArrayObject *)s)->nd) == 0) return 0;
     if(PyString_Check(s)) return stop_at_string ? 0:1;
     if (PySequence_Length(s) == 0) return 1;
 	
@@ -2245,43 +2279,38 @@ static int discover_dimensions(PyObject *s, int nd, int *d) {
 static int array_objecttype(PyObject *op, int minimum_type, int savespaceflag) {
     int l;
     PyObject *ip;
-    PyObject *ap, *arglist;
     int result;
-	
+    
     if (minimum_type == -1) return -1;
 	
     if (PyArray_Check(op)) return max((int)((PyArrayObject *)op)->descr->type_num, minimum_type);
 	
-    if (PyInstance_Check(op)) {
-	if (PyObject_HasAttrString(op, "__array__")) {
-	    ip = PyObject_CallMethod(op, "__array__", NULL);
-            if(!ip) {
-                /* the original code seems to make no provision for the __array__ */
-                /* call to fail. I do this is a fallback, and the */
-                /* actual call to __array__ does get checked elsewhere. */
-                /* maybe ok?  Probably the Python error flag is on at this point*/
-                /* -- Dubois 3/2002 */
-		return (int)PyArray_OBJECT;
-            }
-	    result = max(minimum_type, (int)((PyArrayObject *)ip)->descr->type_num);
-            Py_DECREF(ip);
-            return result;
-	} else {
-	    if (PySequence_Length(op)<0)
-		PyErr_Clear();
-	    return (int)PyArray_OBJECT;
-	}
-    }
+    if (PyObject_HasAttrString(op, "__array__")) {
+        ip = PyObject_CallMethod(op, "__array__", NULL);
+        if(!ip) {
+            /* the original code seems to make no provision for the __array__ */
+            /* call to fail. I do this is a fallback, and the */
+            /* actual call to __array__ does get checked elsewhere. */
+            /* maybe ok?  Probably the Python error flag is on at this point*/
+            /* -- Dubois 3/2002 */
+            return (int)PyArray_OBJECT;
+        }
+        result = max(minimum_type, (int)((PyArrayObject *)ip)->descr->type_num);
+        Py_DECREF(ip);
+        return result;
+    } 
 	
-    if (PyString_Check(op)) {
+    if (PyString_Check(op) || PyUnicode_Check(op)) {
 	return max(minimum_type, (int)PyArray_CHAR);
     }
 	
     if (PySequence_Check(op)) {
 	l = PyObject_Length(op);
+        if (l < 0 && PyErr_Occurred()) { PyErr_Clear(); return (int)PyArray_OBJECT;}
 	if (l == 0 && minimum_type == 0) minimum_type = (savespaceflag ? PyArray_SHORT : PyArray_LONG);
 	while (--l >= 0) {
 	    ip = PySequence_GetItem(op, l);
+	    if (ip==NULL) {PyErr_Clear(); return (int) PyArray_OBJECT;}
 	    minimum_type = array_objecttype(ip, minimum_type, savespaceflag);
 	    Py_DECREF(ip);
 	}
@@ -2313,12 +2342,12 @@ static int Assign_Array(PyArrayObject *self, PyObject *v) {
     }
 	
     l=PyObject_Length(v);
-    if(l < 0) return -1;
+    if(l < 0) return -1; 
 	
     while(--l >= 0)
 	{
-	    e=PySequence_GetItem(v,l);
-	    if (e == NULL) return -1;
+            e=PySequence_GetItem(v,l);
+	    if (e == NULL) return -1; 
 	    r = PySequence_SetItem((PyObject*)self,l,e);
 	    Py_DECREF(e);
 	    if(r == -1) return -1;
@@ -2336,14 +2365,17 @@ Array_FromSequence(PyObject *s, int type, int min_depth, int max_depth)
 	PyErr_SetString(PyExc_ValueError,"expect source sequence");
 	return NULL;
     }
+
     if (!((nd=discover_depth(s,99, type == PyArray_OBJECT || type == 'O')) > 0)) {
 	PyErr_SetString(PyExc_ValueError, "invalid input sequence");
 	return NULL;
     }
+
     if ((max_depth && nd > max_depth) || (min_depth && nd < min_depth)) {
 	PyErr_SetString(PyExc_ValueError, "invalid number of dimensions");
 	return NULL;
     }
+
     if ((d=(int *)malloc(nd*sizeof(int))) == NULL) {
 	PyErr_SetString(PyExc_MemoryError,"out of memory");
     }
@@ -2352,6 +2384,7 @@ Array_FromSequence(PyObject *s, int type, int min_depth, int max_depth)
 	    free(d);
 	    return 0;
 	}
+
 	
     if (type == PyArray_CHAR && nd > 0 && d[nd-1] == 1) {
 	nd = nd-1;
@@ -2384,6 +2417,28 @@ PyObject *PyArray_FromScalar(PyObject *op, int type) {
     }
 }
 
+int PyArray_ValidType(int type) {
+    switch(type) {
+    case 'c': 
+    case 'b': 
+    case '1': 
+    case 's':
+    case 'w': 
+    case 'i':
+    case 'u':
+    case 'l':
+    case 'f': 
+    case 'd': 
+    case 'F': 
+    case 'D': 
+    case 'O': 
+        break;
+    default:
+        if (type >= PyArray_NTYPES) return 0;
+    }
+    return 1;
+}
+
 PyObject *PyArray_Cast(PyArrayObject *mp, int type) {
     PyArrayObject *rp, *tmp;
 	
@@ -2409,7 +2464,6 @@ PyObject *array_fromobject(PyObject *op_in, int type, int min_depth, int max_dep
        The comments try to explain some of the checks. */
 
     PyObject *r, *op;
-    PyObject *ap, *arglist;
     char temp;
     int real_type;
 
@@ -2480,6 +2534,11 @@ PyObject *array_fromobject(PyObject *op_in, int type, int min_depth, int max_dep
     /* The input object is either not an array or it is an object array, 
        or the desired output type is object. */ 
     else {
+	if (PyUnicode_Check(op)) {
+	    PyErr_SetString(PyExc_ValueError, "Unicode objects not supported.");
+	    Py_DECREF(op);
+	    return NULL;
+	}
 	r = Array_FromSequence(op, real_type, min_depth,max_depth);
 	if (r == NULL && min_depth <= 0) {
 	    PyErr_Clear();
@@ -2536,11 +2595,14 @@ extern int PyArray_CanCastSafely(int fromtype, int totype) {
 	return (totype >= PyArray_SHORT);
     case PyArray_SBYTE:
     case PyArray_SHORT:
+    case PyArray_USHORT:
 	return (totype > fromtype);
     case PyArray_INT:
     case PyArray_LONG:
 	/*Shouldn't allow Longs to be cast to Ints, not safe on 64-bit machines!*/
 	return (totype >= PyArray_INT) && (totype != PyArray_FLOAT); 
+    case PyArray_UINT:
+        return (totype >= PyArray_FLOAT); 
     case PyArray_FLOAT:
 	return (totype > PyArray_FLOAT);
     case PyArray_DOUBLE:
@@ -2577,6 +2639,9 @@ extern int PyArray_As2D(PyObject **op, char ***ptr, int *d1, int *d2, int typeco
 	
     n = ap->dimensions[0];
     data = (char **)malloc(n*sizeof(char *));
+    if (!data) {
+        return -1;
+    }
     for(i=0; i<n; i++) {
 	data[i] = ap->data + i*ap->strides[0];
     }
