@@ -10,10 +10,10 @@
 
 */
 
+#include "Python.h"
 #include <string.h>
 #include <math.h>
 
-#include "Python.h"
 #include "Numeric/arrayobject.h"
 
 static PyObject *MultiArrayError;
@@ -21,7 +21,7 @@ static PyObject *MultiArrayError;
 /*Sets the maximum number of dimensions in an array to 40.
   If you ever need to change this I'd love to know more about your arrays.
 */
-#define MAX_DIMS 40
+#define MAX_DIMS 30
 static int compare_lists(int *l1, int *l2, int n) {
     int i;
     for(i=0;i<n;i++) {
@@ -36,7 +36,7 @@ static int compare_lists(int *l1, int *l2, int n) {
 extern PyObject *PyArray_Concatenate(PyObject *op) {
     PyArrayObject *ret, **mps;
     PyObject *otmp;
-    int i, n, type_num, tmp, nd, new_dim;
+    int i, n, type_num, tmp, nd=0, new_dim;
     char *data;
 	
     n = PySequence_Length(op);
@@ -152,7 +152,7 @@ extern PyObject *PyArray_Transpose(PyArrayObject *ap, PyObject *op) {
     long *axes, axis;
     int i, n;
     int *permutation = NULL;
-    PyArrayObject *ret;
+    PyArrayObject *ret = NULL;
 	
     if (op == Py_None) {
       n = ap->nd;
@@ -211,7 +211,7 @@ extern PyObject *PyArray_Transpose(PyArrayObject *ap, PyObject *op) {
 extern PyObject *PyArray_Repeat(PyObject *aop, PyObject *op, int axis) {
     long *counts;
     int n, n_outer, i, j, k, chunk, total, tmp;
-    PyArrayObject *ret, *ap;
+    PyArrayObject *ret=NULL, *ap;
     char *new_data, *old_data;
 	
     ap = (PyArrayObject *)PyArray_ContiguousFromObject(aop, 
@@ -391,14 +391,16 @@ extern PyObject *PyArray_Choose(PyObject *ip, PyObject *op) {
 int fname(type *ip1, type *ip2) { return *ip1 < *ip2 ? -1 : *ip1 == *ip2 ? 0 : 1; }
 
 COMPARE(DOUBLE_compare, double)
-     COMPARE(FLOAT_compare, float)
-     COMPARE(SHORT_compare, short)
-     COMPARE(INT_compare, int)
-     COMPARE(LONG_compare, long)
-     COMPARE(BYTE_compare, signed char)
-     COMPARE(UNSIGNEDBYTE_compare, unsigned char)
+COMPARE(FLOAT_compare, float)
+COMPARE(SHORT_compare, short)
+COMPARE(UNSIGNEDSHORT_compare, unsigned short)
+COMPARE(INT_compare, int)
+COMPARE(UINT_compare, int)
+COMPARE(LONG_compare, long)
+COMPARE(BYTE_compare, signed char)
+COMPARE(UNSIGNEDBYTE_compare, unsigned char)
 
-     int OBJECT_compare(PyObject **ip1, PyObject **ip2) { 
+int OBJECT_compare(PyObject **ip1, PyObject **ip2) { 
     return PyObject_Compare(*ip1, *ip2);
 }
 
@@ -410,7 +412,9 @@ static CompareFunction compare_functions[] =
     (CompareFunction)UNSIGNEDBYTE_compare,
     (CompareFunction)BYTE_compare,
     (CompareFunction)SHORT_compare,
+    (CompareFunction)UNSIGNEDSHORT_compare,
     (CompareFunction)INT_compare,
+    (CompareFunction)UINT_compare,
     (CompareFunction)LONG_compare,
     (CompareFunction)FLOAT_compare,
     (CompareFunction)DOUBLE_compare,
@@ -589,10 +593,12 @@ extern PyObject *PyArray_BinarySearch(PyObject *op1, PyObject *op2) {
 }
 
 DOT_PRODUCT(SHORT_DotProduct, short)
-     DOT_PRODUCT(INT_DotProduct, int)
-     DOT_PRODUCT(LONG_DotProduct, long)
-     DOT_PRODUCT(FLOAT_DotProduct, float)
-     DOT_PRODUCT(DOUBLE_DotProduct, double)
+DOT_PRODUCT(UNSIGNEDSHORT_DotProduct, unsigned short)
+DOT_PRODUCT(INT_DotProduct, int)
+DOT_PRODUCT(UINT_DotProduct, int)
+DOT_PRODUCT(LONG_DotProduct, long)
+DOT_PRODUCT(FLOAT_DotProduct, float)
+DOT_PRODUCT(DOUBLE_DotProduct, double)
 
 #define CDOT_PRODUCT(name, number) \
 	static void name(char *ip1, int is1, char *ip2, int is2, char *op, int n) { \
@@ -611,6 +617,7 @@ DOT_PRODUCT(SHORT_DotProduct, short)
 				   char *op, int n) {
 	 int i;
 	 PyObject *tmp1, *tmp2, *tmp=NULL;
+	 PyObject **tmp3;
 	 for(i=0;i<n;i++,ip1+=is1,ip2+=is2) { 
 	     tmp1 = PyNumber_Multiply(*((PyObject **)ip1), *((PyObject **)ip2));
 	     if (!tmp1) { Py_XDECREF(tmp); return;}
@@ -624,15 +631,29 @@ DOT_PRODUCT(SHORT_DotProduct, short)
 		 tmp = tmp2;
 	     }
 	 } 
-	 Py_XDECREF(*((PyObject **)op));
+         tmp3 = (PyObject**) op;
+         tmp2 = *tmp3;
 	 *((PyObject **)op) = tmp;
+	 Py_XDECREF(tmp2);
      }
 
 typedef void (DotFunction) (char *, int, char *, int, char *, int);
 
-static DotFunction *matrixMultiplyFunctions[] = {NULL,NULL,NULL,SHORT_DotProduct,INT_DotProduct,LONG_DotProduct,
-						 FLOAT_DotProduct,DOUBLE_DotProduct, 
-						 CFLOAT_DotProduct, CDOUBLE_DotProduct, OBJECT_DotProduct};
+static DotFunction *matrixMultiplyFunctions[] = {
+    NULL,
+    NULL,
+    NULL,
+    SHORT_DotProduct,
+    UNSIGNEDSHORT_DotProduct,
+    INT_DotProduct,
+    UINT_DotProduct,
+    LONG_DotProduct,
+    FLOAT_DotProduct,
+    DOUBLE_DotProduct, 
+    CFLOAT_DotProduct, 
+    CDOUBLE_DotProduct, 
+    OBJECT_DotProduct
+};
 
 extern PyObject *PyArray_InnerProduct(PyObject *op1, PyObject *op2) {
     PyArrayObject *ap1, *ap2, *ret;
@@ -732,9 +753,9 @@ extern PyObject *PyArray_MatrixProduct(PyObject *op1, PyObject *op2) {
 	
 	
     ret = NULL;
-    ap1 = (PyArrayObject *)PyArray_FromObject(op1, typenum, 0, 0);
+    ap1 = (PyArrayObject *)PyArray_ContiguousFromObject(op1, typenum, 0, 0);
     if (ap1 == NULL) return NULL;
-    ap2 = (PyArrayObject *)PyArray_FromObject(op2, typenum, 0, 0);
+    ap2 = (PyArrayObject *)PyArray_ContiguousFromObject(op2, typenum, 0, 0);
     if (ap2 == NULL) goto fail;
 	
     if (ap1->nd == 0 || ap2->nd == 0) {
@@ -956,29 +977,38 @@ typedef int (*ArgFunction) (void*, long, long*);
 for(i=1; i<n; i++) { if (ip[i] op mp) { mp = ip[i]; *ap=i; } } return 0;}
 
 ARGFUNC(DOUBLE_argmax, double, >)
-     ARGFUNC(FLOAT_argmax, float, >)
-     ARGFUNC(LONG_argmax, long, >)
-     ARGFUNC(INT_argmax, int, >)
-     ARGFUNC(SHORT_argmax, short, >)
-     ARGFUNC(BYTE_argmax, signed char, >) 
-     ARGFUNC(UNSIGNEDBYTE_argmax, unsigned char, >) 
-     int OBJECT_argmax(PyObject **ip, long n, long *ap) {
+ARGFUNC(FLOAT_argmax, float, >)
+ARGFUNC(LONG_argmax, long, >)
+ARGFUNC(INT_argmax, int, >)
+ARGFUNC(UINT_argmax, int, >)
+ARGFUNC(SHORT_argmax, short, >)
+ARGFUNC(UNSIGNEDSHORT_argmax, unsigned short, >)
+ARGFUNC(BYTE_argmax, signed char, >) 
+ARGFUNC(UNSIGNEDBYTE_argmax, unsigned char, >) 
+
+int OBJECT_argmax(PyObject **ip, long n, long *ap) {
     long i; PyObject *mp=ip[0]; *ap=0;
     for(i=1; i<n; i++) { 
-	if (PyObject_Compare(ip[i],mp) > 0) { mp = ip[i]; *ap=i; } 
+	    if (PyObject_Compare(ip[i],mp) > 0) { mp = ip[i]; *ap=i; } 
     } 
     return 0;
 }
 
-static ArgFunction argmax_functions[] = {NULL,
-                                (ArgFunction) UNSIGNEDBYTE_argmax,
-                                (ArgFunction) BYTE_argmax,
-                                (ArgFunction)SHORT_argmax,
-			        (ArgFunction)INT_argmax,(ArgFunction)LONG_argmax,
-				(ArgFunction)FLOAT_argmax,(ArgFunction)DOUBLE_argmax,
-				NULL,
-				NULL,
-				(ArgFunction)OBJECT_argmax};
+static ArgFunction argmax_functions[] = {
+    NULL,
+    (ArgFunction) UNSIGNEDBYTE_argmax,
+    (ArgFunction) BYTE_argmax,
+    (ArgFunction)SHORT_argmax,
+    (ArgFunction)UNSIGNEDSHORT_argmax,
+    (ArgFunction)INT_argmax,
+    (ArgFunction)UINT_argmax,
+    (ArgFunction)LONG_argmax,
+    (ArgFunction)FLOAT_argmax,
+    (ArgFunction)DOUBLE_argmax,
+    NULL,
+    NULL,
+    (ArgFunction)OBJECT_argmax
+};
 
 extern PyObject *PyArray_ArgMax(PyObject *op) {
     PyArrayObject *ap, *rp;
@@ -1024,7 +1054,9 @@ extern PyObject *PyArray_ArgMax(PyObject *op) {
 	      ARGFUNC(FLOAT_argmin, float, <)
 	      ARGFUNC(LONG_argmin, long, <)
 	      ARGFUNC(INT_argmin, int, <)
+	      ARGFUNC(UINT_argmin, int, <)
 	      ARGFUNC(SHORT_argmin, short, <)
+              ARGFUNC(UNSIGNEDSHORT_argmin, unsigned short, <)
 
 	      int OBJECT_argmin(PyObject **ip, long n, PyObject **mp, long *ap) {
 	      long i; *mp=ip[0]; *ap=0;
@@ -1040,8 +1072,8 @@ static char doc_array[] = "array(sequence, typecode=None, copy=1, savespace=0) w
 static PyObject *
 array_array(PyObject *ignored, PyObject *args, PyObject *kws)
 {
-    PyObject *op, *ret, *tpo=Py_None;
-    char *kwd[]= {"data", "typecode", "copy", "savespace", NULL};
+    PyObject *op, *ret=NULL, *tpo=Py_None;
+    static char *kwd[]= {"data", "typecode", "copy", "savespace", NULL};
     char *tp=NULL;
     int copy=1;
     int savespace=0;
@@ -1060,29 +1092,24 @@ array_array(PyObject *ignored, PyObject *args, PyObject *kws)
 	}
 	if (tp[0] == 0) type = PyArray_NOTYPE;
 	else {
-          type = tp[0];
-          switch(type) {
-          case 'c': 
-          case 'b': 
-          case '1': 
-          case 's': 
-          case 'i': 
-          case 'l':
-          case 'f': 
-          case 'd': 
-          case 'F': 
-          case 'D': 
-          case 'O': 
-            break;
-          default:
-            if (type >= PyArray_NTYPES) {
-              PyErr_SetString(PyExc_TypeError, "typecode argument must be a valid type.");
-              return NULL;
+            type = tp[0];
+            if (!PyArray_ValidType(type)) {
+                PyErr_SetString(PyExc_TypeError, "typecode argument must be a valid type.");
+                return NULL;
             }
-          }
         }
     }
 
+    /* fast exit if simple call */
+    if (PyArray_Check(op) && (copy==0) && 
+	(savespace==PyArray_ISSPACESAVER(op))) {
+	if ((type == PyArray_NOTYPE) || 
+	    (type == ((PyArrayObject *)op)->descr->type_num )) {
+	    Py_INCREF(op);
+	    return op;
+	}
+    }
+
     if (savespace != 0)
 	type |= SAVESPACEBIT;
 
@@ -1100,18 +1127,20 @@ array_array(PyObject *ignored, PyObject *args, PyObject *kws)
     return ret;
 }
 
-static char doc_zeros[] = "zeros((d1,...,dn),typecode,savespace) will return a new array of shape (d1,...,dn) and type typecode (default 'l') with all it's entries initialized to zero.  If savespace (default 0) is nonzero the array will be a spacesaver array.";
+static char doc_zeros[] = "zeros((d1,...,dn),typecode='l',savespace=0) will return a new array of shape (d1,...,dn) and type typecode with all it's entries initialized to zero.  If savespace is nonzero the array will be a spacesaver array.";
 
-static PyObject *array_zeros(PyObject *ignored, PyObject *args) {
+static PyObject *array_zeros(PyObject *ignored, PyObject *args, PyObject *kwds) {
     PyObject *op, *sequence, *tpo=Py_None;
     PyArrayObject *ret;
     char type_char='l';
     char *type = &type_char, *dptr;
     int i, nd, n, dimensions[MAX_DIMS];
     int savespace=0;
-    char all_zero[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
+    static char all_zero[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
+    static char *kwlist[] = {"shape", "typecode", "savespace", NULL};
 	
-    if (!PyArg_ParseTuple(args, "O|Oi", &sequence, &tpo, &savespace)) {
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|Oi", kwlist,
+                                     &sequence, &tpo, &savespace)) {
 	return NULL;
     }
     if (tpo != Py_None) {
@@ -1207,12 +1236,14 @@ static PyObject *array_fromString(PyObject *ignored, PyObject *args, PyObject *k
 
 static char doc_take[] = "take(a, indices, axis=0).  Selects the elements in indices from array a along the given axis.";
 
-static PyObject *array_take(PyObject *dummy, PyObject *args) {
+static PyObject *array_take(PyObject *dummy, PyObject *args, PyObject *kwds) {
     int dimension;
     PyObject *a, *indices, *ret;
+    static char *kwlist[] = {"array", "indices", "axis", NULL};
 	
     dimension=0;
-    if (!PyArg_ParseTuple(args, "OO|i", &a, &indices, &dimension))
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO|i", kwlist, 
+                                     &a, &indices, &dimension))
 	return NULL;
 	
     ret = PyArray_Take(a, indices, dimension);
@@ -1270,11 +1301,13 @@ static PyObject *array_concatenate(PyObject *dummy, PyObject *args) {
 
 static char doc_transpose[] = "transpose(a, axes=None)";
 
-static PyObject *array_transpose(PyObject *dummy, PyObject *args) {
+static PyObject *array_transpose(PyObject *dummy, PyObject *args, PyObject *kwds) {
     PyObject *shape=Py_None, *ret, *a0;
     PyArrayObject *a;
+    static char *kwlist[] = {"array", "axes", NULL};
 	
-    if (!PyArg_ParseTuple(args, "O|O", &a0, &shape)) return NULL;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|O", kwlist, 
+                                     &a0, &shape)) return NULL;
     if ((a = (PyArrayObject *)PyArray_FromObject(a0,
 						 PyArray_NOTYPE,0,0))
 	==NULL) return NULL;
@@ -1287,11 +1320,13 @@ static PyObject *array_transpose(PyObject *dummy, PyObject *args) {
 
 static char doc_repeat[] = "repeat(a, n, axis=0)";
 
-static PyObject *array_repeat(PyObject *dummy, PyObject *args) {
+static PyObject *array_repeat(PyObject *dummy, PyObject *args, PyObject *kwds) {
     PyObject *shape, *a0;
     int axis=0;
+    static char *kwlist[] = {"array", "shape", "axis", NULL};
 	
-    if (!PyArg_ParseTuple(args, "OO|i", &a0, &shape, &axis)) return NULL;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO|i", kwlist, 
+                                     &a0, &shape, &axis)) return NULL;
 	
     return PyArray_Repeat(a0, shape, axis);
 }
@@ -1366,16 +1401,19 @@ static PyObject *array_fastCopyAndTranspose(PyObject *dummy, PyObject *args) {
     return PyArray_fastCopyAndTranspose(a0);
 }
 
-static char doc_correlate[] = "cross_correlate(a,v)";
+static char doc_correlate[] = "cross_correlate(a,v, mode=0)";
 
-static PyObject *array_correlate(PyObject *dummy, PyObject *args) {
+static PyObject *array_correlate(PyObject *dummy, PyObject *args, PyObject *kwds) {
     PyObject *shape, *a0;
     int mode=0;
+    static char *kwlist[] = {"a", "v", "mode", NULL};
 	
-    if (!PyArg_ParseTuple(args, "OO|i", &a0, &shape, &mode)) return NULL;
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO|i", kwlist, 
+                                     &a0, &shape, &mode)) return NULL;
 	
     return PyArray_Correlate(a0, shape, mode);
 }
+
 static char doc_argmax[] = "argmax(a)";
 
 static PyObject *array_argmax(PyObject *dummy, PyObject *args) {
@@ -1393,7 +1431,7 @@ static PyObject *
 array_arange(PyObject *ignored, PyObject *args, PyObject *kws) {
     PyObject *o_start=NULL, *o_stop=Py_None, *o_step=NULL;
     PyObject *tpo=Py_None, *range;
-    char *kwd[]= {"start", "stop", "step", "typecode", NULL};
+    static char *kwd[]= {"start", "stop", "step", "typecode", NULL};
     char *tp=NULL;
     char *rptr;
     int elsize;
@@ -1460,10 +1498,9 @@ array_arange(PyObject *ignored, PyObject *args, PyObject *kws) {
     rptr = ((PyArrayObject *)range)->data;
     elsize = ((PyArrayObject *)range)->descr->elsize;
     type = ((PyArrayObject *)range)->descr->type_num;
-    value = start;
     for (i=0; i < length; i++) {
+        value = start + i*step;
 	dbl_descr->cast[type]((char*)&value, 0, rptr, 0, 1);
-	value += step;
 	rptr += elsize;
     }
     
@@ -1482,42 +1519,46 @@ array_arange(PyObject *ignored, PyObject *args, PyObject *kws) {
       }
 *****/
 
-static char doc_set_string_function[] = "set_string_function(f) sets the python function f to be the function used to obtain a pretty printable string version of a array whenever a array is printed.  f(M) should expect a array argument M, and should return a string consisting of the desired representation of M for printing.";
+static char doc_set_string_function[] = "set_string_function(f, repr=1) sets the python function f to be the function used to obtain a pretty printable string version of a array whenever a array is printed.  f(M) should expect a array argument M, and should return a string consisting of the desired representation of M for printing.";
 
-static PyObject *array_set_string_function(PyObject *dummy, PyObject *args) {
+static PyObject *array_set_string_function(PyObject *dummy, PyObject *args, PyObject *kwds) {
     PyObject *op;
     int repr=1;
-    if(!PyArg_ParseTuple(args, "O|i", &op, &repr)) return NULL; 
+    static char *kwlist[] = {"f", "repr", NULL};
+
+    if(!PyArg_ParseTupleAndKeywords(args, kwds, "O|i", kwlist, 
+                                    &op, &repr)) return NULL; 
     PyArray_SetStringFunction(op, repr);
     Py_INCREF(Py_None);
     return Py_None;
 }
 
 static struct PyMethodDef array_module_methods[] = {
-    {"set_string_function", array_set_string_function, 1, doc_set_string_function},
+    {"set_string_function", (PyCFunction)array_set_string_function, 
+                   METH_VARARGS|METH_KEYWORDS, doc_set_string_function},
 		
-    {"array",	(PyCFunction)array_array, 3, doc_array},
-    {"arange",  (PyCFunction)array_arange, 3, doc_arange},
-    {"zeros",	(PyCFunction)array_zeros, 1, doc_zeros},
-    {"fromstring",(PyCFunction)	array_fromString, 3, doc_fromString},
-    {"take",	(PyCFunction)array_take, 1, doc_take},
-    {"put",	(PyCFunction)array_put, 1, doc_put},
-    {"putmask",	(PyCFunction)array_putmask, 1, doc_putmask},
-    {"reshape",	(PyCFunction)array_reshape, 1, doc_reshape},
-    {"concatenate",	(PyCFunction)array_concatenate, 1, doc_concatenate},
-    {"transpose",	(PyCFunction)array_transpose, 1, doc_transpose},
-    {"repeat",	(PyCFunction)array_repeat, 1, doc_repeat},
-    {"choose",	(PyCFunction)array_choose, 1, doc_choose},
+    {"array",	(PyCFunction)array_array, METH_VARARGS|METH_KEYWORDS, doc_array},
+    {"arange",  (PyCFunction)array_arange, METH_VARARGS|METH_KEYWORDS, doc_arange},
+    {"zeros",	(PyCFunction)array_zeros, METH_VARARGS|METH_KEYWORDS, doc_zeros},
+    {"fromstring",(PyCFunction)array_fromString, METH_VARARGS|METH_KEYWORDS, doc_fromString},
+    {"take",	(PyCFunction)array_take, METH_VARARGS|METH_KEYWORDS, doc_take},
+    {"put",	(PyCFunction)array_put, METH_VARARGS, doc_put},
+    {"putmask",	(PyCFunction)array_putmask, METH_VARARGS, doc_putmask},
+    {"reshape",	(PyCFunction)array_reshape, METH_VARARGS, doc_reshape},
+    {"concatenate",	(PyCFunction)array_concatenate, METH_VARARGS, doc_concatenate},
+    {"transpose",	(PyCFunction)array_transpose, METH_VARARGS|METH_KEYWORDS, doc_transpose},
+    {"repeat",	(PyCFunction)array_repeat, METH_VARARGS|METH_KEYWORDS, doc_repeat},
+    {"choose",	(PyCFunction)array_choose, METH_VARARGS, doc_choose},
 	
-    {"sort",	(PyCFunction)array_sort, 1, doc_sort},
-    {"argsort",	(PyCFunction)array_argsort, 1, doc_argsort},
-    {"binarysearch",	(PyCFunction)array_binarysearch, 1, doc_binarysearch},
+    {"sort",	(PyCFunction)array_sort, METH_VARARGS, doc_sort},
+    {"argsort",	(PyCFunction)array_argsort, METH_VARARGS, doc_argsort},
+    {"binarysearch",	(PyCFunction)array_binarysearch, METH_VARARGS, doc_binarysearch},
 	
-    {"argmax",	(PyCFunction)array_argmax, 1, doc_argmax},
-    {"innerproduct",	(PyCFunction)array_innerproduct, 1, doc_innerproduct}, 
-    {"matrixproduct",	(PyCFunction)array_matrixproduct, 1, doc_matrixproduct}, 
-    {"_fastCopyAndTranspose", (PyCFunction)array_fastCopyAndTranspose, 1, doc_fastCopyAndTranspose},
-    {"cross_correlate", (PyCFunction)array_correlate, 1, doc_correlate},
+    {"argmax",	(PyCFunction)array_argmax, METH_VARARGS, doc_argmax},
+    {"innerproduct",	(PyCFunction)array_innerproduct, METH_VARARGS, doc_innerproduct}, 
+    {"matrixproduct",	(PyCFunction)array_matrixproduct, METH_VARARGS, doc_matrixproduct}, 
+    {"_fastCopyAndTranspose", (PyCFunction)array_fastCopyAndTranspose, METH_VARARGS, doc_fastCopyAndTranspose},
+    {"cross_correlate", (PyCFunction)array_correlate, METH_VARARGS | METH_KEYWORDS, doc_correlate},
     /*  {"arrayMap",	(PyCFunction)array_arrayMap, 1, doc_arrayMap},*/
 	
     {NULL,		NULL, 0}		/* sentinel */
