@@ -102,6 +102,15 @@ class BasicNumericTestCase (unittest.TestCase):
         assert eq(b[2:10], [[1,2,3,4],[5,6,7,8]])
         assert eq(b[2:10, ...], [[1,2,3,4],[5,6,7,8]])
 
+    def testBackwardStrides (self):
+        "Test backward stride operations like x[3::-1]"
+        m = Numeric.array([1.,2.,3.,4.])
+        assert eq(m[2:1:-1], [3.])
+        assert eq(m[2::-1], [3.,2.,1.])
+        assert eq(m[1::-1], [2.,1.])
+        assert eq(m[4::-1], m[3::-1])
+        assert eq(m[4::-2], [4.,2.])
+
     def testOperators (self):
         "Test the operators +, -, *, /, %, ^, &, |"
         x = Numeric.array([1.,2.,3.,4.,5.,6.])
@@ -148,6 +157,14 @@ class BasicNumericTestCase (unittest.TestCase):
         assert eq (x[0], [2,0])
         assert eq (x[1], [0,1])
         assert (4L*Numeric.arange(3)).typecode() == Numeric.PyObject
+        x = Numeric.array([1,2,3,4,5,6],'u')
+        y = Numeric.array([1,2,0,2,2,3],'u')
+        assert eq(x + y, [2, 4, 3, 6, 7, 9])
+        assert eq(x - y, [0, 0, 3, 3, 6, 3])
+        assert eq(x * y, [1, 4, 0, 8, 10, 18])
+        assert eq(y / x, [1, 1, 0, 0, 0, 0])
+        assert eq(y // x, [1, 1, 0, 0, 0, 0])
+        assert eq(x**2, [1, 4, 9, 16, 25, 36])
 
     def testComparisons (self):
         "Test rich comparisons."
@@ -315,7 +332,7 @@ class ArrayFunctionsTestCase (unittest.TestCase):
         "test take"
         x = Numeric.take(self.a, (3,2,1,2,3))
         assert eq (x, [3,2,1,2,3])
-        x = Numeric.take(self.m, (2,2,1,1), -1)
+        x = Numeric.take(self.m, (2,2,1,1), axis=-1)
         assert eq(x, [[3,3,2,2],[13,13,12,12]])
 
     def testReshape (self):
@@ -550,19 +567,64 @@ class ArrayRelationsToOtherModulesTestCase (unittest.TestCase):
         x[1] = 4
         assert not Numeric.allclose(x,c1)
         c2 = copy.copy(y)
-        assert id(c2[1]) == id(x)
+        assert id(c2[1]) == id(Texx)
         c3 = copy.deepcopy(y)
         assert id(c3[1]) != id(x)
         assert Numeric.allclose(c3[1], x)
         
-def suite():
-    testsuite = unittest.makeSuite(BasicNumericTestCase, 'test')
-    testsuite.addTest(unittest.makeSuite(ArrayCreatorsTestCase, 'test'))
-    testsuite.addTest(unittest.makeSuite(ArrayFunctionsTestCase, 'test'))
-    testsuite.addTest(unittest.makeSuite(UniversalFunctionsTestCase, 'test'))
-    testsuite.addTest(unittest.makeSuite(ArrayRelationsToOtherModulesTestCase, 'test'))
-    return testsuite
+class LinearAlgebraTest (unittest.TestCase):
+    def testMatrixMultiply (self):
+        "test matrix multiply"
+# Oct-23 2002 Greg Smith
+# illustrates numpy 'matrixmultiply' bug:
+#  (4x4) *  (4x4x2) yields garbage if the
+# (4x4x2) matrix is constructed with a transpose
+#
+        Mat1 = Numeric.array([[ 1.,  0.,  0.,  0.],
+               [ 0.,  1.,  0.,  0.],
+               [ 0.,  0.,  1.,  0.],
+               [ 0.,  0.,  0.,  1.]])
+        Mat2 = Numeric.array([[[-2.46970211e+002,  1.06732118e+002],
+                [ 1.07496283e+000, -4.64055023e-001],
+                [ 2.25803428e-004, -9.74732723e-005],
+                [ 1.12163980e-007, -4.83802289e-008]],
+               [[ 1.76938633e-001,  1.44333130e+000],
+                [-2.68918870e-004,  4.89638391e-004],
+                [-9.27555680e-008,  1.18379212e-007],
+                [-1.09403911e-010,  8.49856762e-011]],
+               [[-4.83160979e-005, -3.92004807e-004],
+                [ 6.66762717e-008, -2.27579536e-007],
+                [ 2.02732450e-011, -6.23944029e-011],
+                [ 6.46933270e-014, -6.92531088e-014]],
+               [[ 9.60621511e-009,  7.74154579e-008],
+                [-1.26503646e-011,  5.96735496e-011],
+                [-1.32003879e-015,  1.64594378e-014],
+                [-2.02671071e-017,  2.56964410e-017]]])
+        result1 = Numeric.matrixmultiply( Mat1,Mat2 )
+#same thing again, except a matrix identical to Mat2 is
+# constructed by transposing:
+#
+
+        Mat2a = Numeric.transpose(Numeric.array(\
+        [[[-2.46970211e+002,  1.76938633e-001, -4.83160979e-005,  9.60621511e-009],
+        [ 1.07496283e+000, -2.68918870e-004,  6.66762717e-008, -1.26503646e-011],
+        [ 2.25803428e-004, -9.27555680e-008,  2.02732450e-011, -1.32003879e-015],
+        [ 1.12163980e-007, -1.09403911e-010,  6.46933270e-014, -2.02671071e-017]],
+       [[ 1.06732118e+002,  1.44333130e+000, -3.92004807e-004,  7.74154579e-008],
+        [-4.64055023e-001,  4.89638391e-004, -2.27579536e-007,  5.96735496e-011],
+        [-9.74732723e-005,  1.18379212e-007, -6.23944029e-011,  1.64594378e-014],
+        [-4.83802289e-008,  8.49856762e-011, -6.92531088e-014,  2.56964410e-017]]]))
+
+        result2 = Numeric.matrixmultiply( Mat1,Mat2a )
+        self.failUnless(eq(result1, result2))
+
+    def testgeneralizedInverse (self):
+        "Test LinearAlgebra.generalized_inverse"
+        import LinearAlgebra
+        ca = Numeric.array([[1,1-1j],[0,1]])
+        cai = LinearAlgebra.inverse(ca)
+        cai2 = LinearAlgebra.generalized_inverse(ca)
+        self.failUnless(eq(cai, cai2))
 
 if __name__ == '__main__':
-    r = unittest.TextTestRunner()
-    r.run(suite())
+    unittest.main()
